"""
Routes and sub-resources for the /simulation-traces resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
simulation_traces_bp = Blueprint(
    api_constants.MGMT_WEBAPP.SIMULATION_TRACES_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SIMULATION_TRACES_RESOURCE}")


@simulation_traces_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                         api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def simulation_traces():
    """
    The /simulation-traces resource.

    :return: A list of simulation traces or a list of ids of the traces or deletes the traces
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of simulation traces
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return simulation_traces_ids()

        simulation_trcs = MetastoreFacade.list_simulation_traces()
        traces_dicts = list(map(lambda x: x.to_dict(), simulation_trcs))
        response = jsonify(traces_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        traces = MetastoreFacade.list_simulation_traces()
        for trace in traces:
            MetastoreFacade.remove_simulation_trace(trace)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def simulation_traces_ids():
    """
    :return: An HTTP response with all simulation ids
    """
    ids_simulations = MetastoreFacade.list_simulation_traces_ids()
    response_dicts = []
    for tup in ids_simulations:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.SIMULATION_PROPERTY: tup[1]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@simulation_traces_bp.route("/<trace_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                    api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def simulation_trace(trace_id: int):
    """
    The /simulation-traces/id resource.

    :param trace_id: the id of the trace

    :return: The given trace or delets the trace
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    trace = MetastoreFacade.get_simulation_trace(id=trace_id)
    response = jsonify({})
    if trace is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(trace.to_dict())
        else:
            MetastoreFacade.remove_simulation_trace(trace)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /users resource
"""
from flask import Blueprint, jsonify, request
import json
import bcrypt
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util
from csle_common.dao.management.management_user import ManagementUser


# Creates a blueprint "sub application" of the main REST app
users_bp = Blueprint(
    api_constants.MGMT_WEBAPP.USERS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.USERS_RESOURCE}")


@users_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                             api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def users():
    """
    The /users resource.

    :return: A list of management users or a list of ids of the users or deletes all users
    """
    requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of users
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return users_ids()

        users = MetastoreFacade.list_management_users()
        for i in range(len(users)):
            users[i].password = ""
            users[i].salt = ""
        users_dicts = list(map(lambda x: x.to_dict(), users))
        response = jsonify(users_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        users = MetastoreFacade.list_management_users()
        for user in users:
            MetastoreFacade.remove_management_user(management_user=user)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def users_ids():
    """
    :return: An HTTP response with all user ids
    """
    user_ids = MetastoreFacade.list_management_users_ids()
    response_dicts = []
    for tup in user_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@users_bp.route(f"{constants.COMMANDS.SLASH_DELIM}<user_id>",
                methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET, api_constants.MGMT_WEBAPP.HTTP_REST_DELETE,
                         api_constants.MGMT_WEBAPP.HTTP_REST_PUT])
def user(user_id: int):
    """
    The /users/id resource.

    :param user_id: the id of the user

    :return: The given user or deletes the user
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=False)
    if authorized is not None:
        return authorized
    # Check if user is admin or is editing his/hers own account
    user = MetastoreFacade.get_management_user_config(id=user_id)
    request_user = rest_api_util.check_if_user_edit_is_authorized(request=request, user=user)
    if not isinstance(request_user, ManagementUser):
        return request_user

    response = jsonify({})
    if user is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(user.to_dict())
        elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_PUT:
            new_user = json.loads(request.data)[api_constants.MGMT_WEBAPP.USER_PROPERTY]
            new_user = ManagementUser.from_dict(new_user)
            if new_user.password == "":
                new_user.password = user.password
                new_user.salt = user.salt
            else:
                byte_pwd = new_user.password.encode('utf-8')
                salt = bcrypt.gensalt()
                pw_hash = bcrypt.hashpw(byte_pwd, salt)
                new_user.salt = salt.decode("utf-8")
                new_user.password = pw_hash.decode("utf-8")

            # A user cannot be given admin rights by a non-admin user
            if not request_user.admin:
                new_user.admin = False

            # An admin user cannot remove its own admin rights
            if request_user.admin and user.admin and request_user.username == user.username:
                new_user.admin = True
            MetastoreFacade.update_management_user(management_user=new_user, id=user_id)
            new_user.salt = ""
            new_user.password = ""
            response = jsonify(user.to_dict())
        else:
            MetastoreFacade.remove_management_user(management_user=user)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@users_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CREATE_SUBRESOURCE}",
                methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def create_user():
    """
    The /users/create resource.

    :return: creates a new user
    """
    response = jsonify({})
    if constants.CONFIG_FILE.PARSED_CONFIG is not None and constants.CONFIG_FILE.PARSED_CONFIG.allow_registration:
        if request.data is not None:
            json_data = json.loads(request.data)
            if api_constants.MGMT_WEBAPP.USERNAME_PROPERTY in json_data \
                    and api_constants.MGMT_WEBAPP.PASSWORD_PROPERTY in json_data \
                    and api_constants.MGMT_WEBAPP.FIRST_NAME_PROPERTY in json_data \
                    and api_constants.MGMT_WEBAPP.FIRST_NAME_PROPERTY in json_data:
                username = json_data[api_constants.MGMT_WEBAPP.USERNAME_PROPERTY]
                password = json_data[api_constants.MGMT_WEBAPP.PASSWORD_PROPERTY]
                first_name = json_data[api_constants.MGMT_WEBAPP.FIRST_NAME_PROPERTY]
                last_name = json_data[api_constants.MGMT_WEBAPP.LAST_NAME_PROPERTY]
                organization = json_data[api_constants.MGMT_WEBAPP.ORGANIZATION_PROPERTY]
                email = json_data[api_constants.MGMT_WEBAPP.EMAIL_PROPERTY]
                if password == "" or username == "":
                    error_reason = "Password or username cannot be empty."
                    response = jsonify({"reason": error_reason})
                    return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
                else:
                    usernames = list(map(lambda x: x.username, MetastoreFacade.list_management_users()))
                    if username in usernames:
                        error_reason = "A user with that username already exists"
                        response = jsonify({"reason": error_reason})
                        return response, constants.HTTPS.CONFLICT_STATUS_CODE
                    byte_pwd = password.encode('utf-8')
                    salt = bcrypt.gensalt()
                    pw_hash = bcrypt.hashpw(byte_pwd, salt)
                    user = ManagementUser(username=username,
                                          password=pw_hash.decode("utf-8"), admin=False,
                                          salt=salt.decode("utf-8"), first_name=first_name, last_name=last_name,
                                          organization=organization, email=email)
                    MetastoreFacade.save_management_user(management_user=user)
                    response = jsonify(user.to_dict())
                response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
                return response, constants.HTTPS.OK_STATUS_CODE
            else:
                return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
        else:
            return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    else:
        return response, constants.HTTPS.UNAUTHORIZED_STATUS_CODE



"""
Routes and resources for the /traces-page page
"""

from flask import Blueprint
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants


def get_traces_page_bp(static_folder: str) -> Blueprint:
    """
    Creates a blueprint "sub application" of the main REST app that represents the traces page

    :param static_folder: the folder with the static resources
    :return: the created blueprint
    """
    # Creates a blueprint "sub application" of the main REST app
    traces_page_bp = Blueprint(api_constants.MGMT_WEBAPP.TRACES_PAGE_RESOURCE, __name__,
                               url_prefix=f"{constants.COMMANDS.SLASH_DELIM}"
                                          f"{api_constants.MGMT_WEBAPP.TRACES_PAGE_RESOURCE}",
                               static_url_path=f'{constants.COMMANDS.SLASH_DELIM}'
                                               f'{api_constants.MGMT_WEBAPP.TRACES_PAGE_RESOURCE}'
                                               f'{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.STATIC}',
                               static_folder=static_folder)

    @traces_page_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
    def traces_page():
        """
        :return: static resources for the /traces-page url
        """
        return traces_page_bp.send_static_file(api_constants.MGMT_WEBAPP.STATIC_RESOURCE_INDEX)

    return traces_page_bp



class MGMT_WEBAPP:
    """
    Constants related to the management webapp
    """
    ACCESS_CONTROL_ALLOW_ORIGIN_HEADER = "Access-Control-Allow-Origin"
    IDS_QUERY_PARAM = "ids"
    DOWNLOAD_QUERY_PARAM = "download"
    TOKEN_QUERY_PARAM = "token"
    STOP_QUERY_PARAM = "stop"
    EMULATION_QUERY_PARAM = "emulation"
    EXECUTION_ID_QUERY_PARAM = "executionid"
    IP_QUERY_PARAM = "ip"
    CONTAINER_NAME_PROPERTY = "container_name"
    STATIC_RESOURCE_INDEX = "index.html"
    HTTP_REST_GET = "GET"
    HTTP_REST_POST = "POST"
    HTTP_REST_DELETE = "DELETE"
    HTTP_REST_PUT = "PUT"
    STATIC = "static"
    ABOUT_PAGE_RESOURCE = "about-page"
    LOGIN_PAGE_RESOURCE = "login-page"
    REGISTER_PAGE_RESOURCE = "register-page"
    EMULATION_STATISTICS_PAGE_RESOURCE = "emulation-statistics-page"
    EMULATIONS_PAGE_RESOURCE = "emulations-page"
    IMAGES_PAGE_RESOURCE = "images-page"
    DOWNLOADS_PAGE_RESOURCE = "downloads-page"
    JOBS_PAGE_RESOURCE = "jobs-page"
    MONITORING_PAGE_RESOURCE = "monitoring-page"
    POLICIES_PAGE_RESOURCE = "policies-page"
    POLICY_EXAMINATION_PAGE_RESOURCE = "policy-examination-page"
    SDN_CONTROLLERS_PAGE_RESOURCE = "sdn-controllers-page"
    CONTROL_PLANE_PAGE_RESOURCE = "control-plane-page"
    USER_ADMIN_PAGE_RESOURCE = "user-admin-page"
    HOST_TERMINAL_PAGE_RESOURCE = "host-terminal-page"
    CONTAINER_TERMINAL_PAGE_RESOURCE = "container-terminal-page"
    SYSTEM_ADMIN_PAGE_RESOURCE = "system-admin-page"
    LOGS_ADMIN_PAGE_RESOURCE = "logs-admin-page"
    SIMULATIONS_PAGE_RESOURCE = "simulations-page"
    SYSTEM_MODELS_PAGE_RESOURCE = "system-models-page"
    TRACES_PAGE_RESOURCE = "traces-page"
    TRAINING_PAGE_RESOURCE = "training-page"
    CADVISOR_RESOURCE = "cadvisor"
    GRAFANA_RESOURCE = "grafana"
    NODE_EXPORTER_RESOURCE = "node-exporter"
    PROMETHEUS_RESOURCE = "prometheus"
    EMULATIONS_RESOURCE = "emulations"
    EXECUTIONS_SUBRESOURCE = "executions"
    INFO_SUBRESOURCE = "info"
    CLIENT_MANAGER_SUBRESOURCE = "client-manager"
    CLIENT_POPULATION_SUBRESOURCE = "client-population"
    CLIENT_PRODUCER_SUBRESOURCE = "client-producer"
    KAFKA_MANAGER_SUBRESOURCE = "kafka-manager"
    KAFKA_SUBRESOURCE = "kafka"
    ELK_MANAGER_SUBRESOURCE = "elk-manager"
    ELK_STACK_SUBRESOURCE = "elk-stack"
    ELASTIC_SUBRESOURCE = "elastic"
    KIBANA_SUBRESOURCE = "kibana"
    LOGSTASH_SUBRESOURCE = "logstash"
    OSSEC_IDS_MANAGER_SUBRESOURCE = "ossec-ids-manager"
    OSSEC_IDS_SUBRESOURCE = "ossec-ids"
    OSSEC_IDS_MONITOR_SUBRESOURCE = "ossec-ids-monitor"
    SNORT_IDS_MANAGER_SUBRESOURCE = "snort-ids-manager"
    SNORT_IDS_SUBRESOURCE = "snort-ids"
    SNORT_IDS_MONITOR_SUBRESOURCE = "snort-ids-monitor"
    HOST_MANAGER_SUBRESOURCE = "host-manager"
    HOST_MONITOR_SUBRESOURCE = "host-monitor"
    TRAFFIC_MANAGER_SUBRESOURCE = "traffic-manager"
    TRAFFIC_GENERATOR_SUBRESOURCE = "traffic-generator"
    DOCKER_STATS_MANAGER_SUBRESOURCE = "docker-stats-manager"
    DOCKER_STATS_MONITOR_SUBRESOURCE = "docker-stats-monitor"
    CREATE_SUBRESOURCE = "create"
    REGISTRATION_ALLOWED_SUBRESOURCE = "registration-allowed"
    HOST_TERMINAL_ALLOWED_SUBRESOURCE = "host-terminal-allowed"
    CONTAINER_SUBRESOURCE = "container"
    SWITCHES_SUBRESOURCE = "switches"
    MONITOR_SUBRESOURCE = "monitor"
    EMULATION_EXECUTIONS_RESOURCE = "emulation-executions"
    EMULATION_TRACES_RESOURCE = "emulation-traces"
    EMULATION_SIMULATION_TRACES_RESOURCE = "emulation-simulation-traces"
    SIMULATION_TRACES_RESOURCE = "simulation-traces"
    TRACES_DATASETS_RESOURCE = "traces-datasets"
    STATISTICS_DATASETS_RESOURCE = "statistics-datasets"
    SIMULATIONS_RESOURCE = "simulations"
    EMULATION_STATISTICS_RESOURCE = "emulation-statistics"
    IMAGES_RESOURCE = "images"
    FILE_RESOURCE = "file"
    LOGIN_RESOURCE = "login"
    JOBS_RESOURCE = "jobs"
    MONITORING_RESOURCE = "monitoring"
    POLICIES_RESOURCE = "policies"
    SDN_CONTROLLERS_RESOURCE = "sdn-controllers"
    SYSTEM_MODELS_RESOURCE = "system-models"
    GAUSSIAN_MIXTURE_SYSTEM_MODELS_RESOURCE = "gaussian-mixture-system-models"
    EMPIRICAL_SYSTEM_MODELS_RESOURCE = "empirical-system-models"
    GP_SYSTEM_MODELS_RESOURCE = "gp-system-models"
    EXPERIMENTS_RESOURCE = "experiments"
    MULTI_THRESHOLD_POLICIES_RESOURCE = "multi-threshold-policies"
    PPO_POLICIES_RESOURCE = "ppo-policies"
    ALPHA_VEC_POLICIES_RESOURCE = "alpha-vec-policies"
    VECTOR_POLICIES_RESOURCE = "vector-policies"
    TABULAR_POLICIES_RESOURCE = "tabular-policies"
    USERS_RESOURCE = "users"
    LOGS_RESOURCE = "logs"
    CONFIG_RESOURCE = "config"
    VERSION_RESOURCE = "version"
    DQN_POLICIES_RESOURCE = "dqn-policies"
    FNN_W_SOFTMAX_POLICIES_RESOURCE = "fnn-w-softmax-policies"
    TRACES_RESOURCE = "traces"
    TRAINING_RESOURCE = "training"
    RUNNING_PROPERTY = "running"
    REGISTRATION_ALLOWED_PROPERTY = "registration_allowed"
    HOST_TERMINAL_ALLOWED_PROPERTY = "host_terminal_allowed_property"
    VERSION_PROPERTY = "version"
    OUTPUT_PROPERTY = "output"
    INPUT_PROPERTY = "input"
    ROWS_PROPERTY = "rows"
    COLS_PROPERTY = "cols"
    WS_CONNECT_MSG = "connect"
    WS_HOST_TERMINAL_OUTPUT_MSG = "host-terminal-output"
    WS_HOST_TERMINAL_INPUT_MSG = "host-terminal-input"
    WS_RESIZE_MSG = "resize"
    WS_CONTAINER_TERMINAL_OUTPUT_MSG = "container-terminal-output"
    WS_CONTAINER_TERMINAL_INPUT_MSG = "container-terminal-input"
    IP_PROPERTY = "ip"
    EXEC_ID_PROPERTY = "exec_id"
    EMULATION_PROPERTY = "emulation"
    STATISTIC_ID_PROPERTY = "statistic_id"
    SYSTEM_MODEL_TYPE = "system_model_type"
    SIMULATION_PROPERTY = "simulation"
    TRACES_DATASET_PROPERTY = "traces_dataset"
    STATISTICS_DATASET_PROPERTY = "statistics_dataset"
    ID_PROPERTY = "id"
    NAME_PROPERTY = "name"
    SIZE_PROPERTY = "size"
    TRAINING_JOBS_RESOURCE = "training-jobs"
    DATA_COLLECTION_JOBS_RESOURCE = "data-collection-jobs"
    SYSTEM_IDENTIFICATION_JOBS_RESOUCE = "system-identification-jobs"
    LOGS_PROPERTY = "logs"
    PATH_PROPERTY = "path"
    USER_PROPERTY = "user"
    PORT_PROPERTY = "port"
    THREAD_PROPERTY = "thread"
    START_PROPERTY = "start"
    CONFIG_PROPERTY = "config"
    STOP_PROPERTY = "stop"
    STOP_ALL_PROPERTY = "stop-all"
    START_ALL_PROPERTY = "start-all"
    USERNAME_PROPERTY = "username"
    PASSWORD_PROPERTY = "password"
    FIRST_NAME_PROPERTY = "first_name"
    LAST_NAME_PROPERTY = "last_name"
    EMAIL_PROPERTY = "email"
    ORGANIZATION_PROPERTY = "organization"
    TOKEN_PROPERTY = "token"
    ADMIN_PROPERTY = "admin"
    DPID_PROPERTY = "dpid"
    DESC_PROPERTY = "desc"
    FLOWS_PROPERTY = "flows"
    AGG_FLOWS_PROPERTY = "aggflows"
    TABLES_PROPERTY = "tables"
    TABLE_FEATURES_PROPERTY = "tablefeatures"
    PORT_STATS_PROPERTY = "portstats"
    PORT_DESCS_PROPERTY = "portdescs"
    QUEUES_PROPERTY = "queues"
    QUEUE_CONFIGS_PROPERTY = "queueconfigs"
    GROUPS_PROPERTY = "groups"
    GROUP_DESCS_PROPERTY = "groupdescs"
    GROUP_FEATURES_PROPERTY = "groupfeatures"
    METERS_PROPERTY = "meters"
    METER_CONFIGS_PROPERTY = "meter_configs"
    METER_FEATURES_PROPERTY = "meter_features"
    ROLES_PROPERTY = "roles"
    ACTIVE_COUNT_PROPERTY = "active_count"
    TABLE_ID_PROPERTY = "table_id"
    URL_PROPERTY = "url"
    TRAJECTORIES_PROPERTY = "trajectories"
    CONTENT_TYPE_HEADER = "Content-type"
    ACCEPT_HEADER = "Accept"
    APPLICATION_JSON_DATA_TYPE = "application/json"
    GAUSSIAN_MIXTURE_SYSTEM_MODEL_TYPE = "gaussian_mixture"
    EMPIRICAL_SYSTEM_MODEL_TYPE = "empirical"
    GP_SYSTEM_MODEL_TYPE = "gp"
    KIBANA_TUNNELS_DICT = {}
    KIBANA_TUNNEL_BASE_PORT = 17000
    WS_HOST_TERMINAL_NAMESPACE = "host-terminal"
    WS_CONTAINER_TERMINAL_NAMESPACE = "container-terminal"
    HOST_TERMINAL_FD = "host-terminal-fd"
    HOST_TERMINAL_CHILD_PID = "host-terminal-child-pid"
    HOST_TERMINAL_CMD = "host-terminal-cmd"
    CONTAINER_TERMINAL_SSH_SHELL = "container-terminal-ssh-shell"
    CONTAINER_TERMINAL_SSH_CONNECTION = "container-terminal-ssh-connection"


class SESSION_TOKENS:
    """
    Constants related to session tokens
    """
    EXPIRE_TIME_HOURS = 48



"""
Routes and sub-resources for the /system-models resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
system_models_bp = Blueprint(
    api_constants.MGMT_WEBAPP.SYSTEM_MODELS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SYSTEM_MODELS_RESOURCE}")


@system_models_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def system_models():
    """
    The /system-models resource.

    :return: A list of system-models or a list of ids of the models or deletes the models
    """
    authorized = rest_api_util.check_if_user_is_authorized(request=request)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of system models
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return system_models_ids()
        else:
            response = jsonify({})
            response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def system_models_ids():
    """
    :return: An HTTP response with all system models ids
    """
    response_dicts = []
    gaussian_mixture_system_models_ids = MetastoreFacade.list_gaussian_mixture_system_models_ids()
    for tup in gaussian_mixture_system_models_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1],
            api_constants.MGMT_WEBAPP.STATISTIC_ID_PROPERTY: tup[2],
            api_constants.MGMT_WEBAPP.SYSTEM_MODEL_TYPE: api_constants.MGMT_WEBAPP.GAUSSIAN_MIXTURE_SYSTEM_MODEL_TYPE
        })
    empirical_system_models_ids = MetastoreFacade.list_empirical_system_models_ids()
    for tup in empirical_system_models_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1],
            api_constants.MGMT_WEBAPP.STATISTIC_ID_PROPERTY: tup[2],
            api_constants.MGMT_WEBAPP.SYSTEM_MODEL_TYPE: api_constants.MGMT_WEBAPP.EMPIRICAL_SYSTEM_MODEL_TYPE
        })
    gp_system_models_ids = MetastoreFacade.list_gp_system_models_ids()
    for tup in gp_system_models_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1],
            api_constants.MGMT_WEBAPP.STATISTIC_ID_PROPERTY: tup[2],
            api_constants.MGMT_WEBAPP.SYSTEM_MODEL_TYPE: api_constants.MGMT_WEBAPP.GP_SYSTEM_MODEL_TYPE
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /empirical-system-models resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
empirical_system_models_bp = Blueprint(
    api_constants.MGMT_WEBAPP.EMPIRICAL_SYSTEM_MODELS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.EMPIRICAL_SYSTEM_MODELS_RESOURCE}")


@empirical_system_models_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                               api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def empirical_system_models():
    """
    The /empirical-system-models resource.

    :return: A list of system-models or a list of ids of the models or deletes the models
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of empirical
        # system models
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return empirical_system_models_ids()

        models = MetastoreFacade.list_empirical_system_models()
        models_dicts = list(map(lambda x: x.to_dict(), models))
        response = jsonify(models_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        models = MetastoreFacade.list_empirical_system_models()
        for model in models:
            MetastoreFacade.remove_empirical_system_model(model)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def empirical_system_models_ids():
    """
    :return: An HTTP response with all system models ids
    """
    models_ids = MetastoreFacade.list_empirical_system_models_ids()
    response_dicts = []
    for tup in models_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1],
            api_constants.MGMT_WEBAPP.STATISTIC_ID_PROPERTY: tup[2]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@empirical_system_models_bp.route("/<model_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                          api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def empirical_system_model(model_id: int):
    """
    The /system-models/id resource.

    :param model_id: the id of the model

    :return: The given model or deletes the model
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    model = MetastoreFacade.get_empirical_system_model_config(id=model_id)
    response = jsonify({})
    if model is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(model.to_dict())
        else:
            MetastoreFacade.remove_empirical_system_model(empirical_system_model=model)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /system-identification-jobs resource
"""
import time
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
from csle_common.util.emulation_util import EmulationUtil
from csle_common.controllers.management_system_controller import ManagementSystemController
from csle_system_identification.job_controllers.system_identification_job_manager import SystemIdentificationJobManager
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
system_identification_jobs_bp = Blueprint(
    api_constants.MGMT_WEBAPP.SYSTEM_IDENTIFICATION_JOBS_RESOUCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SYSTEM_IDENTIFICATION_JOBS_RESOUCE}")


@system_identification_jobs_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                  api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def system_identification_jobs():
    """
    The /system-identification-jobs resource.

    :return: A list of system-identification-jobs or a list of ids of the jobs or deletes the jobs
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of
        # system identification jobs
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return system_identification_jobs_ids()

        system_identification_jobs = MetastoreFacade.list_system_identification_jobs()
        alive_jobs = []
        for job in system_identification_jobs:
            if EmulationUtil.check_pid(job.pid):
                job.running = True
            alive_jobs.append(job)
        system_identification_jobs_dicts = list(map(lambda x: x.to_dict(), alive_jobs))
        response = jsonify(system_identification_jobs_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        jobs = MetastoreFacade.list_system_identification_jobs()
        for job in jobs:
            ManagementSystemController.stop_pid(job.pid)
            MetastoreFacade.remove_system_identification_job(system_identification_job=job)
        time.sleep(2)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def system_identification_jobs_ids():
    """
    :return: An HTTP response with all system-identification jobs ids
    """
    system_identification_jobs_ids = MetastoreFacade.list_system_identification_jobs_ids()
    response_dicts = []
    for tup in system_identification_jobs_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1],
            api_constants.MGMT_WEBAPP.RUNNING_PROPERTY: EmulationUtil.check_pid(tup[2])
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@system_identification_jobs_bp.route("/<job_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                           api_constants.MGMT_WEBAPP.HTTP_REST_DELETE,
                                                           api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def system_identification_policy(job_id: int):
    """
    The /system-identification-jobs/id resource.

    :param job_id: the id of the policy

    :return: The given policy or deletes the policy
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    job = MetastoreFacade.get_system_identification_job_config(id=job_id)
    response = jsonify({})
    if job is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            if EmulationUtil.check_pid(job.pid):
                job.running = True
            response = jsonify(job.to_dict())
        elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
            ManagementSystemController.stop_pid(job.pid)
            MetastoreFacade.remove_system_identification_job(system_identification_job=job)
            time.sleep(2)
        elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_POST:
            start = True
            stop = request.args.get(api_constants.MGMT_WEBAPP.STOP_QUERY_PARAM)
            if stop is not None and stop:
                start = False
            if start:
                SystemIdentificationJobManager.start_system_identification_job_in_background(
                    system_identification_job=job)
                time.sleep(4)
            else:
                ManagementSystemController.stop_pid(pid=job.pid)
                time.sleep(2)

    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /traces-datasets resource
"""
import os
from flask import Blueprint, jsonify, request, send_from_directory
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
from csle_common.dao.datasets.traces_dataset import TracesDataset
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
traces_datasets_bp = Blueprint(
    api_constants.MGMT_WEBAPP.TRACES_DATASETS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.TRACES_DATASETS_RESOURCE}")


@traces_datasets_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                       api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def traces_datasets():
    """
    The /traces-datasets resource.

    :return: A list of traces datasets or a list of ids of the traces datasets or deletes the datasets
    """
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
        authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
        if authorized is not None:
            return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of traces
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return traces_datasets_ids()

        traces_datasets = MetastoreFacade.list_traces_datasets()
        traces_dicts = list(map(lambda x: x.to_dict(), traces_datasets))
        response = jsonify(traces_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        traces_datasets = MetastoreFacade.list_traces_datasets()
        for traces_dataset in traces_datasets:
            MetastoreFacade.remove_traces_dataset(traces_dataset)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def traces_datasets_ids():
    """
    :return: An HTTP response with all traces datasets ids
    """
    ids_traces_datasets = MetastoreFacade.list_traces_datasets_ids()
    response_dicts = []
    for tup in ids_traces_datasets:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.TRACES_DATASET_PROPERTY: tup[1]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@traces_datasets_bp.route("/<traces_dataset_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                           api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def traces_dataset(traces_dataset_id: int):
    """
    The /traces-datasets/id resource.

    :param traces_dataset_id: the id of the traces dataset
    :return: The given traces dataset or deletes the traces dataset
    """
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
        authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
        if authorized is not None:
            return authorized

    traces_dataset = MetastoreFacade.get_traces_dataset_metadata(id=traces_dataset_id)
    response = jsonify({})
    if traces_dataset is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            # Check if download query parameter is True, then return the file
            download = request.args.get(api_constants.MGMT_WEBAPP.DOWNLOAD_QUERY_PARAM)
            if download is not None and download:
                return download_dataset_file(traces_dataset)
            else:
                response = jsonify(traces_dataset.to_dict())
        else:
            MetastoreFacade.remove_traces_dataset(traces_dataset)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


def download_dataset_file(traces_dataset: TracesDataset):
    """
    Downloads a dataset file

    :param traces_dataset: the dataset file to download
    :return: the HTTP response
    """
    if traces_dataset.file_path is not None and traces_dataset.file_path != "":
        dir_filename = os.path.split(traces_dataset.file_path)
        dir = dir_filename[0]
        filename = dir_filename[1]
        traces_dataset.download_count = traces_dataset.download_count + 1
        MetastoreFacade.update_traces_dataset(traces_dataset=traces_dataset, id=traces_dataset.id)
        try:
            return send_from_directory(dir, filename, as_attachment=True)
        except FileNotFoundError:
            response = jsonify({})
            return response, constants.HTTPS.NOT_FOUND_STATUS_CODE
    else:
        response = jsonify({})
        return response, constants.HTTPS.NOT_FOUND_STATUS_CODE



"""
Routes and sub-resources for the /dqn-policies resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
dqn_policies_bp = Blueprint(
    api_constants.MGMT_WEBAPP.DQN_POLICIES_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.DQN_POLICIES_RESOURCE}")


@dqn_policies_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                    api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def dqn_policies():
    """
    The /dqn-policies resource.

    :return: A list of dqn-policies or a list of ids of the policies or deletes the policies
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of DQN policies
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return dqn_policies_ids()

        dqn_policies = MetastoreFacade.list_dqn_policies()
        dqn_policies_dicts = list(map(lambda x: x.to_dict(), dqn_policies))
        response = jsonify(dqn_policies_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        policies = MetastoreFacade.list_dqn_policies()
        for policy in policies:
            MetastoreFacade.remove_dqn_policy(dqn_policy=policy)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def dqn_policies_ids():
    """
    :return: An HTTP response with all dqn policies ids
    """
    dqn_policies_ids = MetastoreFacade.list_dqn_policies_ids()
    response_dicts = []
    for tup in dqn_policies_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.SIMULATION_PROPERTY: tup[1]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response


@dqn_policies_bp.route("/<policy_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def dqn_policy(policy_id: int):
    """
    The /dqn-policies/id resource.

    :param policy_id: the id of the policy

    :return: The given policy or deletes the policy
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    policy = MetastoreFacade.get_dqn_policy(id=policy_id)
    response = jsonify({})
    if policy is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(policy.to_dict())
        else:
            MetastoreFacade.remove_dqn_policy(dqn_policy=policy)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response



"""
Routes and sub-resources for the /images resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.controllers.container_controller import ContainerController
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
images_bp = Blueprint(
    api_constants.MGMT_WEBAPP.IMAGES_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.IMAGES_RESOURCE}")


@images_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def images():
    """
    The /images resource

    :return: Returns a list of images
    """
    authorized = rest_api_util.check_if_user_is_authorized(request=request)
    if authorized is not None:
        return authorized

    images = ContainerController.list_all_images()
    images_dicts = []
    for img in images:
        images_dicts.append(
            {
                api_constants.MGMT_WEBAPP.NAME_PROPERTY: img[0],
                api_constants.MGMT_WEBAPP.SIZE_PROPERTY: img[4]
            }
        )
    response = jsonify(images_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /config resource
"""
from flask import Blueprint, jsonify, request
import json
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
import csle_rest_api.util.rest_api_util as rest_api_util
from csle_common.dao.emulation_config.config import Config
from csle_common.logging.log import Logger


# Creates a blueprint "sub application" of the main REST app
config_bp = Blueprint(
    api_constants.MGMT_WEBAPP.CONFIG_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CONFIG_RESOURCE}")


@config_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                              api_constants.MGMT_WEBAPP.HTTP_REST_PUT])
def config():
    """
    The /config resource.

    :return: The CSLE configuration
    """
    requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        try:
            config = Config.read_config_file()
            response = jsonify(config.to_param_dict())
        except Exception as e:
            Logger.__call__().get_logger().info(f"There was an error reading the config file: {str(e)}, {repr(e)}")
            response = jsonify({})
            return response, constants.HTTPS.INTERNAL_SERVER_ERROR_STATUS_CODE
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_PUT:
        json_data = json.loads(request.data)
        # Verify payload
        if api_constants.MGMT_WEBAPP.CONFIG_RESOURCE not in json_data:
            return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
        config = json_data[api_constants.MGMT_WEBAPP.CONFIG_PROPERTY]
        config = Config.from_param_dict(config)
        Config.save_config_file(config=config.to_dict())
        Config.set_config_parameters_from_config_file()
        response = jsonify(config.to_param_dict())
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


@config_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.REGISTRATION_ALLOWED_SUBRESOURCE}",
                 methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def registration_allowed():
    """
    The /config/registration-allowed resource.

    :return: The CSLE configuration
    """
    allow_registration = False
    if constants.CONFIG_FILE.PARSED_CONFIG is not None and constants.CONFIG_FILE.PARSED_CONFIG.allow_registration:
        allow_registration = True
    response_dict = {}
    response_dict[api_constants.MGMT_WEBAPP.REGISTRATION_ALLOWED_PROPERTY] = allow_registration
    response = jsonify(response_dict)
    return response, constants.HTTPS.OK_STATUS_CODE


@config_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.HOST_TERMINAL_ALLOWED_SUBRESOURCE}",
                 methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def host_terminal_allowed():
    """
    The /config/host-terminal-allowed resource.

    :return: The CSLE configuration
    """
    allow_host_terminal = False
    if constants.CONFIG_FILE.PARSED_CONFIG is not None and constants.CONFIG_FILE.PARSED_CONFIG.allow_host_shell:
        allow_host_terminal = True
    response_dict = {}
    response_dict[api_constants.MGMT_WEBAPP.HOST_TERMINAL_ALLOWED_PROPERTY] = allow_host_terminal
    response = jsonify(response_dict)
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /version resource
"""
from flask import Blueprint, jsonify
from importlib.metadata import version as meta_version
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants


# Creates a blueprint "sub application" of the main REST app
version_bp = Blueprint(
    api_constants.MGMT_WEBAPP.VERSION_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.VERSION_RESOURCE}")


@version_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def version():
    """
    The /version resource.

    :return: The version of CSLE management system
    """
    csle_version = meta_version('csle_rest_api')
    csle_version_dict = {}
    csle_version_dict[api_constants.MGMT_WEBAPP.VERSION_PROPERTY] = csle_version
    response = jsonify(csle_version_dict)
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /file resource
"""
import json
import os
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
file_bp = Blueprint(
    api_constants.MGMT_WEBAPP.FILE_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.FILE_RESOURCE}")


@file_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def read_file():
    """
    The /file resource

    :return: Reads a given file and returns its contents
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_POST:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    path = json.loads(request.data)[api_constants.MGMT_WEBAPP.PATH_PROPERTY]
    data = ""
    if os.path.exists(path):
        with open(path, 'r') as fp:
            data = fp.read()
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /logs resource
"""
from flask import Blueprint, jsonify, request
import os
import subprocess
import json
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
import csle_collector.constants.constants as collector_constants
import csle_rest_api.util.rest_api_util as rest_api_util
from csle_common.dao.emulation_config.config import Config
from csle_common.metastore.metastore_facade import MetastoreFacade
from csle_common.util.emulation_util import EmulationUtil


# Creates a blueprint "sub application" of the main REST app
logs_bp = Blueprint(
    api_constants.MGMT_WEBAPP.LOGS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.LOGS_RESOURCE}")


@logs_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def logs():
    """
    The /logs resource.

    :return: List of log files in the CSLE logging directory
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    config = Config.get_current_confg()
    path = config.default_log_dir
    log_files = []
    for f in os.listdir(path):
        item = os.path.join(path, f)
        if os.path.isfile(item):
            log_files.append(item)
    if len(log_files) > 20:
        log_files = log_files[0:20]
    data = log_files
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.DOCKER_STATS_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def docker_stats_manager_logs():
    """
    The /logs/docker-stats-manager resource.

    :return: The logs of the docker stats manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    config = Config.get_current_confg()
    path = config.docker_stats_manager_log_dir + config.docker_stats_manager_log_file

    if os.path.exists(path):
        with open(path, 'r') as fp:
            data = fp.readlines()
            tail = data[-100:]
            data = tail
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.PROMETHEUS_RESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def prometheus_logs():
    """
    The /logs/prometheus resource.

    :return: The logs of the docker stats manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    config = Config.get_current_confg()
    path = config.prometheus_log_file

    if os.path.exists(path):
        with open(path, 'r') as fp:
            data = fp.readlines()
            tail = data[-100:]
            data = tail
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.NODE_EXPORTER_RESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def node_exporter_logs():
    """
    The /logs/node-exporter resource.

    :return: The logs of the docker stats manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    config = Config.get_current_confg()
    path = config.node_exporter_log_file

    if os.path.exists(path):
        with open(path, 'r') as fp:
            data = fp.readlines()
            tail = data[-100:]
            data = tail
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CADVISOR_RESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def cadvisor_logs():
    """
    The /logs/cadvisor resource.

    :return: The logs of the docker stats manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    cmd = constants.COMMANDS.CADVISOR_LOGS
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    output = output.decode("utf-8")
    output = output.split("\n")[-100:]
    data = output
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.GRAFANA_RESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
def grafana_logs():
    """
    The /lofgs/grafana resource.

    :return: The logs of the docker stats manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    cmd = constants.COMMANDS.GRAFANA_LOGS
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    output = output.decode("utf-8")
    output = output.split("\n")[-100:]
    data = output
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CONTAINER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def container_logs():
    """
    The /logs/container resource.

    :return: The logs of a specific container
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized
    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    container_name = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]
    cmd = constants.COMMANDS.CONTAINER_LOGS.format(container_name)
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    output = output.decode("utf-8")
    output = output.split("\n")[-100:]
    data = output
    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
    response = jsonify(data_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CLIENT_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def client_manager_logs():
    """
    The /logs/client-manager resource.

    :return: The logs of the client manager
    """
    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized
    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.traffic_config.client_population_config.client_manager_log_dir +
                emulation_env_config.traffic_config.client_population_config.client_manager_log_file)
        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.KAFKA_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def kafka_manager_logs():
    """
    The /logs/kafka-manager resource.

    :return: The logs of the kafka manager
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.kafka_config.kafka_manager_log_dir +
                emulation_env_config.kafka_config.kafka_manager_log_file)

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.KAFKA_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def kafka_logs():
    """
    The /logs/kafka resource.

    :return: The logs of the kafka server
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = collector_constants.LOG_FILES.KAFKA_LOG_FILE

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SNORT_IDS_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def snort_ids_manager_logs():
    """
    The /logs/snort-ids-manager resource.

    :return: The logs of a Snort IDS manager with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.snort_ids_manager_config.snort_ids_manager_log_dir +
                emulation_env_config.snort_ids_manager_config.snort_ids_manager_log_file)

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SNORT_IDS_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def snort_ids_logs():
    """
    The /logs/snort-ids-logs resource.

    :return: The logs of a Snort IDS with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config

        path = collector_constants.SNORT_IDS_ROUTER.SNORT_FAST_LOG_FILE

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        cmd = f"{constants.COMMANDS.TAIL} -200 {path}"
        o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
        data = []
        for line in o.decode().split("\n"):
            a_str = line.replace("\n", "")
            data.append(a_str)

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.OSSEC_IDS_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def ossec_ids_manager_logs():
    """
    The /logs/ossec-ids-manager resource.

    :return: The logs of a OSSEC IDS manager with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.ossec_ids_manager_config.ossec_ids_manager_log_dir +
                emulation_env_config.ossec_ids_manager_config.ossec_ids_manager_log_file)

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.OSSEC_IDS_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def ossec_ids_logs():
    """
    The /logs/ossec-ids resource.

    :return: The logs of a OSSEC IDS with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = collector_constants.OSSEC.OSSEC_LOG_FILE

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        cmd = f"{constants.COMMANDS.SUDO} {constants.COMMANDS.CHMOD_U_RWX} {path}"
        o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.HOST_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def host_ids_manager_logs():
    """
    The /logs/host-manager resource.

    :return: The logs of a Host manager with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.host_manager_config.host_manager_log_dir +
                emulation_env_config.host_manager_config.host_manager_log_file)
        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.TRAFFIC_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def traffic_manager_logs():
    """
    The /logs/traffic-manager resource.

    :return: The logs of a Traffic manager with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.traffic_config.get_node_traffic_config_by_ip(ip=ip).traffic_manager_log_dir +
                emulation_env_config.traffic_config.get_node_traffic_config_by_ip(ip=ip).traffic_manager_log_file)

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.ELK_MANAGER_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def elk_manager_logs():
    """
    The /logs/elk-manager resource.

    :return: The logs of a ELK manager with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = (emulation_env_config.elk_config.elk_manager_log_dir +
                emulation_env_config.elk_config.elk_manager_log_file)

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE


@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.ELK_STACK_SUBRESOURCE}",
               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def elk_logs():
    """
    The /logs/elk-stack resource.

    :return: The logs of an ELK-stack instance with a specific IP
    """

    # Check that token is valid
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
    if authorized is not None:
        return authorized

    if api_constants.MGMT_WEBAPP.NAME_PROPERTY not in json.loads(request.data):
        response = jsonify({})
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE
    ip = json.loads(request.data)[api_constants.MGMT_WEBAPP.NAME_PROPERTY]

    emulation = request.args.get(api_constants.MGMT_WEBAPP.EMULATION_QUERY_PARAM)
    execution_id = request.args.get(api_constants.MGMT_WEBAPP.EXECUTION_ID_QUERY_PARAM)
    if emulation is not None and ip is not None and execution_id is not None:
        emulation_env_config = MetastoreFacade.get_emulation_execution(ip_first_octet=execution_id,
                                                                       emulation_name=emulation).emulation_env_config
        path = collector_constants.ELK.ELK_LOG

        # Connect
        EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
        sftp_client = emulation_env_config.get_connection(ip=ip).open_sftp()
        remote_file = sftp_client.open(path)
        data = []
        try:
            data = remote_file.read()
            data = data.decode()
            data = data.split("\n")
            data = data[-100:]
        finally:
            remote_file.close()

        data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
        response = jsonify(data_dict)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    else:
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.BAD_REQUEST_STATUS_CODE



"""
Routes and sub-resources for the /emulation-statistics resource
"""
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
import csle_rest_api.util.rest_api_util as rest_api_util

# Creates a blueprint "sub application" of the main REST app
emulation_statistics_bp = Blueprint(
    api_constants.MGMT_WEBAPP.EMULATION_STATISTICS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.EMULATION_STATISTICS_RESOURCE}")


@emulation_statistics_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                            api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def emulation_statistics():
    """
    The /emulation-statistics resource.

    :return: A list of emulation statistics or a list of ids of the statistics or deletes the statistics
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        # Check if ids query parameter is True, then only return the ids and not the whole list of emulation statistics
        ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
        if ids is not None and ids:
            return emulation_statistics_ids()

        stats = MetastoreFacade.list_emulation_statistics()
        stats_dicts = list(map(lambda x: x.to_dict(), stats))
        response = jsonify(stats_dicts)
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        statistics = MetastoreFacade.list_emulation_statistics()
        for stat in statistics:
            MetastoreFacade.remove_emulation_statistic(stat)
        response = jsonify({})
        response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
        return response, constants.HTTPS.OK_STATUS_CODE


def emulation_statistics_ids():
    """
    :return: An HTTP response with all emulation statistics ids
    """
    stats_ids = MetastoreFacade.list_emulation_statistics_ids()
    response_dicts = []
    for tup in stats_ids:
        response_dicts.append({
            api_constants.MGMT_WEBAPP.ID_PROPERTY: tup[0],
            api_constants.MGMT_WEBAPP.EMULATION_PROPERTY: tup[1]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@emulation_statistics_bp.route("/<statistics_id>", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                            api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def emulation_statistic(statistics_id: int):
    """
    The /emulation-statistics/id resource.

    :param statistics_id: the id of the statistic

    :return: The given statistic or deletes the statistic
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    statistic = MetastoreFacade.get_emulation_statistic(id=statistics_id)
    response = jsonify({})
    if statistic is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(statistic.to_dict())
        else:
            MetastoreFacade.remove_emulation_statistic(statistic)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



"""
Routes and sub-resources for the /prometheus resource
"""

from flask import Blueprint, jsonify, request
from csle_common.controllers.management_system_controller import ManagementSystemController
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
prometheus_bp = Blueprint(api_constants.MGMT_WEBAPP.PROMETHEUS_RESOURCE, __name__,
                          url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.PROMETHEUS_RESOURCE}")


@prometheus_bp.route("",
                     methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET, api_constants.MGMT_WEBAPP.HTTP_REST_POST])
def prometheus():
    """
    :return: static resources for the /prometheus url
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_POST:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    running = ManagementSystemController.is_prometheus_running()
    port = constants.COMMANDS.PROMETHEUS_PORT
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_POST:
        if running:
            ManagementSystemController.stop_prometheus()
            running = False
        else:
            ManagementSystemController.start_prometheus()
            running = True
    prometheus_ip = "localhost"
    if constants.CONFIG_FILE.PARSED_CONFIG is not None:
        prometheus_ip = constants.CONFIG_FILE.PARSED_CONFIG.metastore_ip
        port = constants.CONFIG_FILE.PARSED_CONFIG.prometheus_port
    prometheus_dict = {
        api_constants.MGMT_WEBAPP.RUNNING_PROPERTY: running,
        api_constants.MGMT_WEBAPP.PORT_PROPERTY: port,
        api_constants.MGMT_WEBAPP.URL_PROPERTY: f"http://{prometheus_ip}:{port}/"
    }
    response = jsonify(prometheus_dict)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE



from flask import request
import csle_rest_api.util.rest_api_util as rest_api_util
from flask_socketio import ConnectionRefusedError
from flask import Blueprint
import paramiko
import csle_rest_api.constants.constants as api_constants
import csle_common.constants.constants as constants
from csle_rest_api import socketio


def get_container_terminal_bp(app):
    """
    Gets the blue print of the Web socket API for the container terminal

    :param app: the Flask app
    :return: the blue print
    """

    def ssh_connect(ip: str) -> paramiko.SSHClient:
        """
        Sets up an SSH connection to a given IP using the CSLE admin account

        :param ip: the IP to connect to
        :return: the created ssh connection
        """
        conn = paramiko.SSHClient()
        conn.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        conn.connect(ip, username=constants.CSLE_ADMIN.SSH_USER, password=constants.CSLE_ADMIN.SSH_PW)
        conn.get_transport().set_keepalive(5)
        return conn

    def read_and_forward_container_terminal_output() -> None:
        """
        Reads output from a given file descriptor and sends the output to the web socket

        :return: None
        """
        max_read_bytes = 1024 * 20
        while True:
            socketio.sleep(0.01)
            ssh_channel = app.config[api_constants.MGMT_WEBAPP.CONTAINER_TERMINAL_SSH_SHELL]
            data_ready = ssh_channel.recv_ready()
            if data_ready:
                output = ssh_channel.recv(max_read_bytes).decode(errors="ignore")
                socketio.emit(api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_OUTPUT_MSG,
                              {api_constants.MGMT_WEBAPP.OUTPUT_PROPERTY: output},
                              namespace=f"{constants.COMMANDS.SLASH_DELIM}"
                                        f"{api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_NAMESPACE}")

    @socketio.on(api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_INPUT_MSG,
                 namespace=f"{constants.COMMANDS.SLASH_DELIM}"
                           f"{api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_NAMESPACE}")
    def container_terminal_input(data) -> None:
        """
        Receives input msg on a websocket and writes it to the PTY representing the bash shell
        of the Container terminal.
        The pty sees this as if you are typing in a real terminal.

        :param data: the input data to write
        :return: None
        """
        cmd = data[api_constants.MGMT_WEBAPP.INPUT_PROPERTY].encode()
        ssh_channel = app.config[api_constants.MGMT_WEBAPP.CONTAINER_TERMINAL_SSH_SHELL]
        ssh_channel.send(cmd)

    @socketio.on(api_constants.MGMT_WEBAPP.WS_RESIZE_MSG,
                 namespace=f"{constants.COMMANDS.SLASH_DELIM}"
                           f"{api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_NAMESPACE}")
    def container_terminal_resize(data) -> None:
        """
        Handler when receiving a message on a websocket to resize the PTY window of a container terminal.
        The handler parses the data and resizes the window accordingly.

        :param data: data with information about the new PTY size
        :return: None
        """
        pass

    @socketio.on(api_constants.MGMT_WEBAPP.WS_CONNECT_MSG,
                 namespace=f"{constants.COMMANDS.SLASH_DELIM}"
                           f"{api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_NAMESPACE}")
    def container_terminal_connect() -> None:
        """
        Handler for new websocket connection requests for the /container-terminal namespace.

        First checks if the user is authorized and then sets up the connection

        :return: None
        """
        authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
        if authorized is not None or (constants.CONFIG_FILE.PARSED_CONFIG is None):
            raise ConnectionRefusedError()
        ip_str = request.args.get(api_constants.MGMT_WEBAPP.IP_QUERY_PARAM)
        if ip_str is not None:
            ip = ip_str.replace("-", ".")
            term = u'xterm'
            ssh_conn = ssh_connect(ip=ip)
            ssh_channel = ssh_conn.invoke_shell(term=term)
            ssh_channel.setblocking(0)
            app.config[api_constants.MGMT_WEBAPP.CONTAINER_TERMINAL_SSH_SHELL] = ssh_channel
            app.config[api_constants.MGMT_WEBAPP.CONTAINER_TERMINAL_SSH_CONNECTION] = ssh_conn
            socketio.start_background_task(target=read_and_forward_container_terminal_output)
        else:
            ConnectionRefusedError()

    container_terminal_bp = Blueprint(api_constants.MGMT_WEBAPP.WS_CONTAINER_TERMINAL_NAMESPACE, __name__)
    return container_terminal_bp



"""
Routes and sub-resources for the /simulations resource
"""
import base64
from flask import Blueprint, jsonify, request
import csle_common.constants.constants as constants
import csle_rest_api.constants.constants as api_constants
from csle_common.metastore.metastore_facade import MetastoreFacade
from csle_common.controllers.simulation_env_controller import SimulationEnvController
import csle_rest_api.util.rest_api_util as rest_api_util


# Creates a blueprint "sub application" of the main REST app
simulations_bp = Blueprint(
    api_constants.MGMT_WEBAPP.SIMULATIONS_RESOURCE, __name__,
    url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.SIMULATIONS_RESOURCE}")


@simulations_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET, api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def simulations():
    """
    The /simulations resource

    :return: Returns a list of simulations, a list of simulation ids, or deletes all simulations
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    # Check if ids query parameter is True, then only return the ids and not the whole list of simulations
    ids = request.args.get(api_constants.MGMT_WEBAPP.IDS_QUERY_PARAM)
    if ids is not None and ids:
        return simulation_ids()
    all_simulations = MetastoreFacade.list_simulations()
    all_images = MetastoreFacade.list_simulation_images()
    simulations_dicts = {}
    for sim in all_simulations:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            for sim_name_img in all_images:
                sim_name, img = sim_name_img
                if sim_name == sim.name:
                    sim.image = base64.b64encode(img).decode()
        elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
            SimulationEnvController.uninstall_simulation(sim)

    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
        simulations_dicts = list(map(lambda x: x.to_dict(), all_simulations))
    elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        simulations_dicts = {}
    response = jsonify(simulations_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


def simulation_ids():
    """
    :return: the list of simulation ids as an HTTP response
    """
    simulation_ids = MetastoreFacade.list_simulation_ids()
    response_dicts = []
    for tup in simulation_ids:
        response_dicts.append({
            "id": tup[0],
            "simulation": tup[1]
        })
    response = jsonify(response_dicts)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE


@simulations_bp.route('/<simulation_id>', methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET,
                                                   api_constants.MGMT_WEBAPP.HTTP_REST_DELETE])
def get_simulation(simulation_id: int):
    """
    The /simulations/id resource. Gets or delets a simulation with a given id.

    :param simulation_id: the id of the simulation
    :return: the simulation or deletes the simulation
    """
    requires_admin = False
    if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
        requires_admin = True
    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=requires_admin)
    if authorized is not None:
        return authorized

    simulation = MetastoreFacade.get_simulation(simulation_id)
    sim_name_img = MetastoreFacade.get_simulation_image(simulation_name=simulation.name)
    sim_name, img = sim_name_img
    simulation.image = base64.b64encode(img).decode()
    response = jsonify({})
    if simulation is not None:
        if request.method == api_constants.MGMT_WEBAPP.HTTP_REST_GET:
            response = jsonify(simulation.to_dict())
        elif request.method == api_constants.MGMT_WEBAPP.HTTP_REST_DELETE:
            SimulationEnvController.uninstall_simulation(simulation)
    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
    return response, constants.HTTPS.OK_STATUS_CODE
